﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace MaskSlap
{
    public class AudioHandler : MonoBehaviour
    {
        public static AudioHandler _instance { get; private set; }

        private List<AudioSource> availableAudioSources = new List<AudioSource>();
        private List<AudioSource> beingUsedAudioSources = new List<AudioSource>();

        [Header("SFX")]
        [SerializeField]
        private Transform sfxAudioSourcesTranform;
        [SerializeField]
        private SFXData[] sfxData;

        [Header("Footsteps")]
        [SerializeField]
        private AudioSource footstepsSource;

        [SerializeField]
        private float masterVolume = 0.2f;

        private void Awake()
        {
            _instance = this;
        }

        private void Start()
        {
            availableAudioSources.Clear();
            beingUsedAudioSources.Clear();
            for (int i = 0; i < sfxAudioSourcesTranform.childCount; i++)
            {
                availableAudioSources.Add(sfxAudioSourcesTranform.GetChild(i).GetComponent<AudioSource>());
            }
        }

        public void PlayFootsteps(bool play)
        {
            if (play)
            {
                footstepsSource.Play();
            }
            else 
            {
                footstepsSource.Stop();
            }
        }

        public void PlayBGM(bool play)
        {
            if (play)
            {
                //bgmSource.Play();
            }
            else
            {
                //bgmSource.Stop();
            }
        }

        public void PlaySFX(AUDIO_SFX_TYPE type)
        {
            PoolBackUsedAudioSources();

            if (availableAudioSources.Count <= 0)
            {
                Debug.LogError("No free audio source to play sfx clip");
                return;
            }

            AudioSource tempSource = availableAudioSources[0];
            var sfxData = GetAudioClipForSFX(type);
            tempSource.clip = sfxData.clip;
            tempSource.volume = sfxData.GetVolume() * masterVolume;
            tempSource.pitch = sfxData.GetPitch();
            tempSource.Play();
            availableAudioSources.Remove(tempSource);
            beingUsedAudioSources.Add(tempSource);
        }

        private SFXData GetAudioClipForSFX(AUDIO_SFX_TYPE type)
        {
            for (int i = 0; i < sfxData.Length; i++)
            {
                if (sfxData[i].sfx == type)
                {
                    return sfxData[i];
                }
            }

            return null;
        }

        private void PoolBackUsedAudioSources()
        {
            for (int i = beingUsedAudioSources.Count - 1; i >= 0; i--)
            {
                if (!beingUsedAudioSources[i].isPlaying)
                {
                    availableAudioSources.Add(beingUsedAudioSources[i]);
                    beingUsedAudioSources.RemoveAt(i);
                }
            }
        }

        [System.Serializable]
        public class SFXData
        {
            public AUDIO_SFX_TYPE sfx;
            public AudioClip clip;

            public float minVolume;
            public float maxVolume;
            public float minPitch;
            public float maxPitch;

            public float GetVolume()
            {
                return Random.Range(minVolume, maxVolume);
            }
            public float GetPitch()
            {
                return Random.Range(minPitch, maxPitch);
            }
        }
    }

    public enum AUDIO_SFX_TYPE
    {
        DASH,
        ATTACK,
        ATTACKMISS
    }
}