﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class EnemyAnimationHandler : MonoBehaviour
{
    [SerializeField]
    SpriteRenderer spriteRenderer;
    [SerializeField]
    List<Sprite> GettingHitSprites = new List<Sprite>();
    [SerializeField]
    List<Sprite> RunningSprites = new List<Sprite>();
    [SerializeField]
    List<Sprite> MaskedRunningSprites = new List<Sprite>();
    [SerializeField]
    Sprite idle;
    [SerializeField]
    float frameDelay = 0.25f;

    Coroutine GettingCaughtRoutine, RunningRoutine, MaskedRunningRoutine;
    public void PlayGettingHit()
    {
        if (GettingCaughtRoutine != null)
            return;
        if (RunningRoutine != null)
            StopCoroutine(RunningRoutine);
        RunningRoutine = null;
        GettingCaughtRoutine = StartCoroutine(_CR_GettingHitRoutine());
    }
    public void PlayIdle()
    {
        spriteRenderer.sprite = idle;
    }
    public void PlayRunning()
    {
        if (RunningRoutine != null)
            return;
        if (GettingCaughtRoutine != null)
            StopCoroutine(GettingCaughtRoutine);
        GettingCaughtRoutine = null;
        RunningRoutine= StartCoroutine(_CR_RunningRoutine());
    }
    public void PlayMaskedRunning()
    {
        if (RunningRoutine != null)
            StopCoroutine(RunningRoutine);
        if (GettingCaughtRoutine != null)
            StopCoroutine(GettingCaughtRoutine);
        GettingCaughtRoutine = null;
        RunningRoutine = null;
        MaskedRunningRoutine = StartCoroutine(_CR_MaskedRunningRoutine());
    }
    IEnumerator _CR_GettingHitRoutine()
    {
        for (int i = 0; i < GettingHitSprites.Count; i++)
        {
            spriteRenderer.sprite = GettingHitSprites[i];
            yield return new WaitForSecondsRealtime(frameDelay);
        }
    }
    IEnumerator _CR_RunningRoutine()
    {
        int i = 0;
        while(true)
        {
            spriteRenderer.sprite = RunningSprites[i];
            i++;
            if (i == RunningSprites.Count)
                i = 0;
            yield return new WaitForSeconds(frameDelay);
        }
    }
    IEnumerator _CR_MaskedRunningRoutine()
    {
        for (int i = 0; i < MaskedRunningSprites.Count; i++)
        {
            spriteRenderer.sprite = MaskedRunningSprites[i];
            yield return new WaitForSeconds(frameDelay);
        }
    }
}
