﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace MaskSlap
{
    public class GameState : MonoBehaviour
    {
        public static GameState instance;

        private void Awake()
        {
            instance = this;
        }

        private void Start()
        {
            StartCoroutine(LevelLoaderCR());
        }

        private void Update()
        {
#if UNITY_EDITOR
            if (Input.GetKey(KeyCode.LeftShift) && Input.GetKeyDown(KeyCode.W))
            {
                GameWon();
            }
#endif
        }

        private IEnumerator LevelLoaderCR()
        {
            LevelLoader.instance.LoadLevel();

            yield return new WaitUntil(() => LevelLoader.instance.isLevelDoneLoading);

            InitUI();
            InitGameState();

            yield return new WaitForEndOfFrame();

            StartLevel();
        }

        private int slapsLandedScoreCounter;

        #region Game state
        private List<Politician> allPoliticians;
        private List<Guard> allGuards;
        private Player player;

        private void InitGameState()
        {
            allPoliticians = new List<Politician>(FindObjectsOfType<Politician>());
            foreach (var politician in allPoliticians)
            {
                politician.OnHit.AddListener(PoliticianHit);
            }
            allGuards = new List<Guard>(FindObjectsOfType<Guard>());
            slapsLandedScoreCounter = 0;
            player = FindObjectOfType<Player>();

            HUDHandler.instance.UpdateSlapScoreOnHUD(slapsLandedScoreCounter, allPoliticians.Count);
        }

        private void PoliticianHit(Politician pol)
        {
            slapsLandedScoreCounter++;
            HUDHandler.instance.UpdateSlapScoreOnHUD(slapsLandedScoreCounter, allPoliticians.Count);

            //Temp
            pol.GotSlapped();

            CheckForGameWinSituation();
        }

        private void CheckForGameWinSituation()
        {
            if (slapsLandedScoreCounter >= allPoliticians.Count)
            {
                GameWon();
            }
        }

        private void GameWon()
        {
            HUDHandler.instance.ShowGameEndScreen(true);

            foreach (var pol in allPoliticians)
            {
                pol.GameOver();
            }

            foreach (var guard in allGuards)
            {
                guard.GameOver();
            }

            player.GameWon();
            StartCoroutine(LevelEndCR(true));
        }

        public void PlayerCaptured()
        {
            HUDHandler.instance.ShowGameEndScreen(false);
            player.GotCaptured();
            StartCoroutine(LevelEndCR(false));
        }

        private IEnumerator LevelEndCR(bool won)
        {
            yield return new WaitForSeconds(2);

            if (LevelLoader.instance.AllLevelsCompleted() || !won)
            {
                LevelLoader.instance.LoadMainMenu();
            }
            else
            {
                StartCoroutine(LevelLoaderCR());
            }
        }
        #endregion

        #region UI
        // Start playing on UI / Input
        public void StartLevel()
        {
            CameraController._camera.Init();
            Player._player.StartPlaying();
            //AudioHandler._instance.PlayBGM(true);
            HUDHandler.instance.ShowHUD();
        }

        private void InitUI()
        {
            //show hud if needed.
        }
        #endregion
    }
}
