﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;

namespace MaskSlap
{
    public class InputHandler : MonoBehaviour
    {
        public static InputHandler _instance { get; private set; }

#if UNITY_EDITOR
        [SerializeField]
        private Vector2 movement;
        [SerializeField]
        private Vector2 movementRaw;
        [SerializeField]
        private bool isDashHeld;
        [SerializeField]
        private bool isAttackHeld;
#endif

        private InputScheme inputScheme;

        public delegate void InputButtonPressEvent();
        public event InputButtonPressEvent OnDashEvent;
        public event InputButtonPressEvent OnAttackEvent;

        private void Awake()
        {
            _instance = this;
            inputScheme = new InputScheme();
        }

        private void OnEnable()
        {
            inputScheme.Player.Enable();
            SubscribeToInputSchemeEvents();
        }

        private void OnDisable()
        {
            UnSubscribeToInputSchemeEvents();
            inputScheme.Player.Disable();
        }

        private void SubscribeToInputSchemeEvents()
        {
            UnSubscribeToInputSchemeEvents();
            inputScheme.Player.Dash.performed += OnDashPerformed;
            inputScheme.Player.Attack.performed += OnAttackPerformed;
        }

        private void UnSubscribeToInputSchemeEvents()
        {
            inputScheme.Player.Dash.performed -= OnDashPerformed;
            inputScheme.Player.Attack.performed -= OnAttackPerformed;
        }

        private void OnDashPerformed(InputAction.CallbackContext callback)
        {
            OnDashEvent?.Invoke();
        }

        private void OnAttackPerformed(InputAction.CallbackContext callback)
        {
            OnAttackEvent?.Invoke();
        }

        private void Update()
        {
            DebugValues();
        }

        public Vector2 GetMovementAxis()
        {
            Vector2 movementAxis = inputScheme.Player.Movement.ReadValue<Vector2>();
            return movementAxis;
        }

        public Vector2 GetMovementAxisRaw()
        {
            Vector2 movementAxis = new Vector2(Input.GetAxisRaw("Horizontal"), Input.GetAxisRaw("Vertical"));
            return movementAxis;
        }

        public bool IsDashButtonHeld()
        {
            return inputScheme.Player.Dash.phase == InputActionPhase.Performed;
        }

        public bool IsAttackButtonHeld()
        {
            return inputScheme.Player.Attack.phase == InputActionPhase.Performed;
        }

        private void DebugValues()
        {
#if UNITY_EDITOR
            movement = GetMovementAxis();
            movementRaw = GetMovementAxisRaw();
            isDashHeld = IsDashButtonHeld();
            isAttackHeld = IsAttackButtonHeld();
#endif
        }
    }
}
