﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

namespace MaskSlap
{
    public class LevelLoader : MonoBehaviour
    {
        public static LevelLoader instance;

        [HideInInspector]
        public bool isLevelDoneLoading;

        [SerializeField]
        private string[] levelScenes;

        private int levelNumber = 0;

        private void Awake()
        {
            instance = this;
        }

        public bool AllLevelsCompleted()
        {
            return levelNumber >= levelScenes.Length;
        }

        public void LoadMainMenu()
        {
            SceneManager.LoadScene(1);
        }

        public void LoadLevel()
        {
            StartCoroutine(LoadLevelCR());
        }

        private IEnumerator LoadLevelCR()
        {
            isLevelDoneLoading = false;

            AsyncOperation asyncLoad = SceneManager.LoadSceneAsync(levelScenes[levelNumber], LoadSceneMode.Additive);

            while (!asyncLoad.isDone)
            {
                yield return null;
            }

            //Unload last level.
            if (levelNumber > 0)
            {
                var async = SceneManager.UnloadSceneAsync(levelScenes[levelNumber - 1]);
                while (!async.isDone)
                {
                    yield return null;
                }
            }

            levelNumber++;

            isLevelDoneLoading = true;
        }
    }
}
