﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerAnimationHandler : MonoBehaviour
{
    public static PlayerAnimationHandler _instance { get; private set; }

    [SerializeField]
    SpriteRenderer playerSpriteRenderer;

    [SerializeField]
    List<Sprite> RunningSprites = new List<Sprite>(10);
    [SerializeField]
    List<Sprite> HitSprites = new List<Sprite>(10);
    [SerializeField]
    List<Sprite> DashSprites = new List<Sprite>(10);
    [SerializeField]
    Sprite Idle;
    [SerializeField]
    Sprite GettingCaught;
    [SerializeField]
    float runningFrameDelay = 0.25f;
    [SerializeField]
    float hittingFrameDelay = 0.25f;

    private void Awake()
    {
        _instance = this;
    }

    Coroutine LeftRunningRoutine, RightRunningRoutine, HitRoutine, DashRoutine, GettingCaughtRoutine;

    public void PlayMovementAnimations(Vector3 dir)
    {
        if (dir == Vector3.zero)
        {
            PlayIdle();
        }
        else
        {
            if (HitRoutine != null)
                return;
            if (dir.x > 0)
                PlayRunningRight();
            else if (dir.x < 0)
                PlayRunningLeft();
            else if (dir.z > 0)
                PlayRunningRight();
            else
                PlayRunningLeft();
        }
    }
    public void PlayerHitAnimation()
    {
        if (LeftRunningRoutine != null)
        {
            StopCoroutine(LeftRunningRoutine);
            LeftRunningRoutine = null;
        }
        if (RightRunningRoutine != null)
        {
            StopCoroutine(RightRunningRoutine);
            RightRunningRoutine = null;
        }
        if (HitRoutine != null)
            return;
        HitRoutine = StartCoroutine(_CR_HitRoutine());
    }
    public void PlayerDashAnimation()
    {
        if (LeftRunningRoutine != null)
        {
            StopCoroutine(LeftRunningRoutine);
            LeftRunningRoutine = null;
        }
        if (RightRunningRoutine != null)
        {
            StopCoroutine(RightRunningRoutine);
            RightRunningRoutine = null;
        }
        if (HitRoutine != null)
        {
            StopCoroutine(HitRoutine);
            HitRoutine = null;
        }
        if (DashRoutine != null)
            return;
        DashRoutine = StartCoroutine(_CR_DashRoutine());
    }
    public void PlayGettingCaught()
    {
        playerSpriteRenderer.sprite = GettingCaught;
    }

    private void PlayRunningLeft()
    {
        if (LeftRunningRoutine != null)
            return;
        playerSpriteRenderer.flipX = false;
        if (RightRunningRoutine != null)
        {
            StopCoroutine(RightRunningRoutine);
            RightRunningRoutine = null;
        }
        LeftRunningRoutine = StartCoroutine(_CR_RunningRoutine());
    }
    private void PlayRunningRight()
    {
        if (RightRunningRoutine != null)
            return;
        playerSpriteRenderer.flipX = true;
        if (LeftRunningRoutine != null)
        {
            StopCoroutine(LeftRunningRoutine);
            LeftRunningRoutine = null;
        }
        RightRunningRoutine = StartCoroutine(_CR_RunningRoutine());
    }
    private void PlayIdle()
    {
        if (LeftRunningRoutine != null)
        {
            StopCoroutine(LeftRunningRoutine);
            LeftRunningRoutine = null;
        }
        if (RightRunningRoutine != null)
        {
            StopCoroutine(RightRunningRoutine);
            RightRunningRoutine = null;
        }
        playerSpriteRenderer.sprite = Idle;
    }

    IEnumerator _CR_RunningRoutine()
    {
        int index = 0;

        while (true)
        {
            playerSpriteRenderer.sprite = RunningSprites[index];
            index++;
            if (index == RunningSprites.Count)
                index = 0;
            yield return new WaitForSeconds(runningFrameDelay);
        }
    }
    IEnumerator _CR_HitRoutine()
    {
        for(int i = 0; i< HitSprites.Count; i++)
        {
            playerSpriteRenderer.sprite = HitSprites[i];
            yield return new WaitForSeconds(hittingFrameDelay);
        }
        HitRoutine = null;
    }
    IEnumerator _CR_DashRoutine()
    {
        for (int i = 0; i < HitSprites.Count; i++)
        {
            playerSpriteRenderer.sprite = HitSprites[i];
            yield return new WaitForSeconds(hittingFrameDelay);
        }
        HitRoutine = null;
    }
}
