﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

namespace MaskSlap
{
    [RequireComponent(typeof(Collider))]
    public class Sensor : MonoBehaviour
    {
        public class SensorEvent : UnityEvent<Collider> { }

        [SerializeField]
        private bool requireLineOfSight = false;
        [SerializeField]
        private LayerMask raycastMask;
        private int raycastMaskValue;

        [SerializeField]
        private string targetTag = "Player";

        public SensorEvent OnSensorEnter = new SensorEvent();
        public SensorEvent OnSensorStay = new SensorEvent();
        public SensorEvent OnSensorExit = new SensorEvent();

        private void Awake()
        {
            raycastMaskValue = raycastMask.value;
        }

        private void OnTriggerEnter(Collider other)
        {
            if (other.CompareTag(targetTag))
            {
                if (requireLineOfSight && !HasLineOfSight(other))
                {
                    return;
                }
                OnSensorEnter.Invoke(other);
            }
        }

        private void OnTriggerStay(Collider other)
        {
            if (other.CompareTag(targetTag))
            {
                if (requireLineOfSight && !HasLineOfSight(other))
                {
                    return;
                }
                OnSensorStay.Invoke(other);
            }
        }

        private void OnTriggerExit(Collider other)
        {
            if (other.CompareTag(targetTag))
            {
                OnSensorExit.Invoke(other);
            }
        }

        private bool HasLineOfSight(Collider other)
        {
            var dir = other.transform.position - transform.position;
            if (Physics.Raycast(transform.position, dir.normalized, out var hitInfo, 100f, raycastMaskValue))
            {
                if (hitInfo.collider.gameObject == other.gameObject)
                {
                    return true;
                }
                else
                {
                    Debug.Log($"Hit {hitInfo.collider.gameObject.name} instead of {other.gameObject.name}");
                }
            }
            return false;
        }
    }
}
